/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: 2 axis stepper positioning
* Version		: v0.2
* Last updated	: 15.06.08
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "lib_stepper.h"
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>



// ********************* local definitions *********************
		struct axis{				//all private data for X and Y axes
		   uint16_t   Pos;			//current position
  volatile uint8_t	  State;		//status flags
		   int8_t	  Speed;      	//current speed (raw)
  volatile uint16_t   StpRate;	    //step rate
		   };
		   
		struct axis X;
		struct axis Y;	

enum {NXT};							//state flags


const uint8_t Step[] PROGMEM =	{						//halfsteps
									0b00010001,
									0b00110011,
									0b00100010,
									0b01100110,
									0b01000100,
									0b11001100,
									0b10001000,
									0b10011001,
									0b00010001
									};


const uint16_t Ramp[] PROGMEM = {								//acceleration, deceleration (step durations)
27145, 20829, 17560, 15471, 13987, 12862, 11972, 11244, 10635, 10115, 9665,  9270,  8920,  8606, 
8324,  8068,  7834,  7619,  7421,  7238,  7067,  6908,  6760,  6620,  6489,  6366,  6249,  6138, 
6033,  5933,  5839,  5748,  5662,  5579,  5500,  5424,  5351,  5281,  5214,  5149,  5087,  5026, 
4968,  4912,  4858,  4805,  4754,  4705,  4657,  4611,  4566,  4522	
};

#define RAMP_LEN (50)



// *************** Stepper Initialisation ****************
void init_stepper(void)
{
//Ports
DDRA  = 0xFF;													//PortA is low output
PORTA = 0;

//postion reset (homing)
uint8_t tmp, tmpA= 0, tmpB= 0;
uint16_t delay;
X.Pos= (X_MAX +20);
Y.Pos= (Y_MAX +20);
while ((X.Pos != 0) || (Y.Pos != 0))
	{
	if (X.Pos > 0)
		{
		if (PIND &(1<<PD4))
			{
			tmpA= pgm_read_byte(&Step[(--X.Pos &7)]);			//move through full range (open loop)
			tmpA &= 0b00001111;
			}
		else X.Pos= 0;											//Anschlag detected (closed loop reference)
		}
	if (Y.Pos > 0)
		{
		if (PIND &(1<<PD5))
			{
			tmpB= pgm_read_byte(&Step[(--Y.Pos &7)]);
			tmpB &= 0b11110000;
			}
		else Y.Pos= 0;
		}
	tmp= tmpA;
	tmp |= tmpB;
	PORTA= tmp;
	delay= (0xFF -ACC);
	while (delay != 0)
		{
		_delay_us(32);
		delay--;
		} 
	}


//Timer
TCCR1A = 0;
TCCR1B = (1<<CS11);												//set T1 @clock/8
TIMSK |= (1<<OCIE1A)|(1<<OCIE1B)|(1<<TOIE1);					//enable compare interrupts (motor speeds)
}


// *********** acceleration control ***********
void check_accel(struct axis* Stp, uint16_t Target)
{
if (abs(Target -Stp->Pos) <= abs(Stp->Speed))				//decel speed short before target position
	{
	if      (Stp->Speed < 0) Stp->Speed++;
	else if (Stp->Speed > 0) Stp->Speed--; 				
	}

else if (Stp->Pos < Target)									//shall move in positive direction
	{
	if (Stp->Speed < RAMP_LEN) Stp->Speed++; 						//accel speed
	}

else if (Stp->Pos > Target)									//shall move in negative direction
	{
	if (Stp->Speed > -RAMP_LEN) Stp->Speed--; 						//accel speed
	}

if      (Stp->Speed > 0) Stp->Pos++;
else if (Stp->Speed < 0) Stp->Pos--;

uint32_t tmp= (uint32_t)pgm_read_word(&Ramp[abs(Stp->Speed)]);
Stp->StpRate= (tmp * (0xFF -ACC))>>8;
}





// *********** step control (call periodically in main!) *********
void check_step(void)
{
if (X.State &(1<<NXT))
	{
	X.State &=  ~(1<<NXT);
	check_accel(&X, Xtarget);
	uint8_t step= pgm_read_byte(&Step[(X.Pos &7)]);			//get new X pattern
	step &= 0b00001111;
	uint8_t tmp= PORTA &0b11110000;							//get Y pattern
	step |= tmp;											//merge
	PORTA= step;
	}

if (Y.State &(1<<NXT))
	{
	Y.State &=  ~(1<<NXT);
	check_accel(&Y, Ytarget);
	uint8_t step= pgm_read_byte(&Step[(Y.Pos &7)]);			//get new Y pattern
	step &= 0b11110000;
	uint8_t tmp= PORTA &0b00001111;							//get X pattern
	step |= tmp;											//merge
	PORTA= step;
	}
}



// *************** ISRs ****************
ISR (TIMER1_COMPA_vect)										//X axis
{
X.State |= (1<<NXT);
OCR1A += X.StpRate;											//set next step time
}

ISR (TIMER1_COMPB_vect)										//Y axis
{
Y.State |= (1<<NXT);
OCR1B += Y.StpRate;											//set next step time
}


